@@ -2,6 +2,7 @@ from __future__ import print_function
 from .logexception import LogException
 import six
 import json
+import os
 from concurrent.futures import ProcessPoolExecutor, as_completed
 import multiprocessing
 from .logresponse import LogResponse
@@ -16,6 +17,7 @@ from aliyun.log.pulllog_response import PullLogResponse
 from .consumer import *
 from multiprocessing import RLock
 from .util import base64_encodestring as b64e
+import copy
 
 
 MAX_INIT_SHARD_COUNT = 200
@@ -129,7 +131,7 @@ def copy_project(from_client, to_client, from_project, to_project, copy_machine_
             break
 
 
-def copy_logstore(from_client, from_project, from_logstore, to_logstore, to_project=None, to_client=None):
+def copy_logstore(from_client, from_project, from_logstore, to_logstore, to_project=None, to_client=None, to_region_endpoint=None):
     """
     copy logstore, index, logtail config to target logstore, machine group are not included yet.
     the target logstore will be crated if not existing
@@ -152,14 +154,20 @@ def copy_logstore(from_client, from_project, from_logstore, to_logstore, to_proj
     :type to_client: LogClient
     :param to_client: logclient instance, use it to operate on the "to_project" if being specified
 
+    :type to_region_endpoint: string
+    :param to_region_endpoint: target region, use it to operate on the "to_project" while "to_client" not be specified
+
     :return:
     """
 
-    # check client
-    if to_project is not None:
-        # copy to a different project in different client
+    if to_region_endpoint is not None and to_client is None:
+        to_client = copy.deepcopy(from_client)
+        to_client.set_endpoint(to_region_endpoint)
+    else:
         to_client = to_client or from_client
 
+    # check client
+    if to_project is not None:
         # check if target project exists or not
         ret = from_client.get_project(from_project)
         try:
@@ -172,7 +180,6 @@ def copy_logstore(from_client, from_project, from_logstore, to_logstore, to_proj
                 raise
 
     to_project = to_project or from_project
-    to_client = to_client or from_client
 
     # return if logstore are the same one
     if from_client is to_client and from_project == to_project and from_logstore == to_logstore:
@@ -340,14 +347,17 @@ def dump_worker(client, project_name, logstore_name, from_time, to_time,
     try:
         for data in res:
             for log in data.get_flatten_logs_json(decode_bytes=True):
-                with open(file_path, "a+") as f:
+                with open(os.path.expanduser(file_path), "a+") as f:
                     count += 1
                     try:
                         if six.PY2:
                             last_ex = None
                             for encoding in encodings:
                                 try:
-                                    f.write(json.dumps(log, encoding=encoding, ensure_ascii=ensure_ansi))
+                                    ret = json.dumps(log, encoding=encoding, ensure_ascii=ensure_ansi)
+                                    if isinstance(ret, unicode):
+                                        ret = ret.encode(encoding, errors="ignore")
+                                    f.write(ret)
                                     f.write("\n")
                                     break
                                 except UnicodeDecodeError as ex:
@@ -846,16 +856,12 @@ def transform_data(from_client, from_project, from_logstore, from_time,
         cg_data_fetch_interval = cg_data_fetch_interval or 2
         cg_in_order = False if cg_in_order is None else cg_in_order
         cg_worker_pool_size = cg_worker_pool_size or 3
-        cursor_position = CursorPosition.SPECIAL_TIMER_CURSOR
-        if from_time.lower() == "begin":
-            cursor_position = CursorPosition.BEGIN_CURSOR
-        elif from_time.lower() == "end":
-            cursor_position = CursorPosition.END_CURSOR
-        cursor_start_time = from_time
 
         option = LogHubConfig(from_client._endpoint, from_client._accessKeyId, from_client._accessKey,
                               from_project, from_logstore, cg_name,
-                              c_name, cursor_position=cursor_position, cursor_start_time=cursor_start_time,
+                              c_name, cursor_position=CursorPosition.SPECIAL_TIMER_CURSOR,
+                              cursor_start_time=from_time,
+                              cursor_end_time=to_time,
                               heartbeat_interval=cg_heartbeat_interval, data_fetch_interval=cg_data_fetch_interval,
                               in_order=cg_in_order,
                               worker_pool_size=cg_worker_pool_size)
